/***************************************************************************
 *
 * $Header: /home/werner/ytree/RCS/freesp.c,v 1.13 1998/11/11 19:02:48 werner Rel $
 *
 * Ermittlung der freien Plattenkapazitaet
 *
 ***************************************************************************/


#include "ytree.h"

#if ( defined( linux ) || defined( sun ) || defined( __NeXT__ )  || defined(hpux) || defined ( __FreeBSD__ ) || defined( __DJGPP__ ) ) && !defined( SVR4 )

#include <sys/vfs.h>
#else
#ifdef WIN32
#include <dos.h>
#else
#if defined( SVR4 ) || defined( OSF1 )
#include <sys/statvfs.h>
#else
#if ( defined( ultrix ) || defined ( __FreeBSD__ ) )
#include <sys/param.h>
#include <sys/mount.h>
#else
/* z.B. SVR3 */
#include <sys/statfs.h>
#endif /* ultrix */
#endif /* SVR4 */
#endif /* WIN32 */
#endif /* sun / linux / __NeXT__  hpux */

#define MINIMUM( a, b ) ( (a) < (b) ? (a) : (b) )




/* Volume-Name und freien Plattenplatz ermitteln */
/*-----------------------------------------------*/

int GetDiskParameter( char *path, 
		      char *volume_name, 
		      unsigned long *avail_bytes,
		      unsigned long *total_disk_space
		    )
{

#ifdef WIN32
  struct _diskfree_t diskspace;
#else
#if defined( SVR4 ) || defined( OSF1 )
  struct statvfs statfs_struct;
#else
#ifdef ultrix
  struct fs_data statfs_struct;
#else
  struct statfs statfs_struct;
#endif /* ultrix */
#endif /* SVR4 */
#endif /* WIN32 */

  char *p;
  char *fname;
  int  result;
  long bfree;
  unsigned long disk_space;


#ifdef WIN32
  if( ( result = _getdiskfree( 0, &diskspace ) ) == 0 )
#else
#ifdef __DJGPP__
  if( ( result = statfs( path, &statfs_struct ) ) == 0 )
#else

  if( ( result = STATFS( path, &statfs_struct, sizeof( statfs_struct ), 0 ) ) == 0 )
#endif /* __DJGPP__ */
#endif /* WIN32 */
  {
    if( volume_name )
    {
      /* Name ermitteln */
      /*----------------*/

      if( mode == DISK_MODE )
      {
  
#ifdef linux
	switch( statfs_struct.f_type ) {
	  case 0xEF51:
	       fname = "EXT2-OLD"; break;
	  case 0xEF53:
	       fname = "EXT2"; break;
	  case 0x137D:
	       fname = "EXT"; break;
	  case 0x9660:
	       fname = "ISOFS"; break;
	  case 0x137F:
	       fname = "MINIX"; break;
	  case 0x138F:
	       fname = "MINIX2"; break;
	  case 0x2468:
	       fname = "MINIX-NEW"; break;
	  case 0x4d44:
	       fname = "DOS"; break;
	  case 0x6969:
	       fname = "NFS"; break;
	  case 0x9fa0:
	       fname = "PROC"; break;
	  case 0x012FD16D:
	       fname = "XIAFS"; break;
	  default:
	       fname = "LINUX";
	}
#else
#if defined( sun ) || defined( hpux ) || defined( __NeXT__ ) || defined( ultrix ) || defined ( __FreeBSD__ )
        fname = "UNIX";
#else
#ifdef WIN32
        fname = "WIN-NT";
#else
#ifdef __DJGPP__
        fname = "DJGPP";
#else
#if defined( SVR4 ) || defined( OSF1 )
        fname = statfs_struct.f_fstr;
#else
        fname = statfs_struct.f_fname;
#endif /* SVR4 */
#endif /* __DJGPP__ */
#endif /* WIN32 */
#endif /* sun / hpux / __NeXT__ ultrix */
#endif /* linux */

        (void) strncpy( volume_name, 
	                fname,
		        MINIMUM( DISK_NAME_LENGTH, strlen( fname ) )
		      );
        volume_name[ MINIMUM( DISK_NAME_LENGTH, strlen( fname ))] = '\0';
      }
      else
      {  
        /* TAR/ZOO/ZIP-FILE_MODE */
        /*-----------------------*/
        
        if( ( p = strrchr( statistic.login_path, FILE_SEPARATOR_CHAR ) ) == NULL ) 
          p = statistic.login_path;
        else p++;
  
        (void) strncpy( volume_name, p, sizeof( statistic.disk_name ) );
        volume_name[sizeof( statistic.disk_name )] = '\0';
      }
    } /* volume_name */

#ifdef WIN32
    *avail_bytes = (long) diskspace.bytes_per_sector *
                   (long) diskspace.sectors_per_cluster *
                   (long) diskspace.avail_clusters;
    disk_space = 900000000L; /* for now.. */
#else

#if (defined( SVR4 ) || defined( OSF1 )) && !defined( __DGUX__ )
    bfree = getuid() ? statfs_struct.f_bavail : statfs_struct.f_bfree;
    if( bfree < 0L ) bfree = 0L;
    *avail_bytes = bfree * statfs_struct.f_frsize;
    disk_space   = statfs_struct.f_blocks * statfs_struct.f_frsize;
#else
#if defined( _IBMR2 ) || defined( linux ) || defined( sun ) || defined( __NeXT__ )
    bfree = getuid() ? statfs_struct.f_bavail : statfs_struct.f_bfree;
    if( bfree < 0L ) bfree = 0L;
    *avail_bytes = bfree * statfs_struct.f_bsize;
    disk_space   = statfs_struct.f_blocks * statfs_struct.f_blocks;
#else
#ifdef SVR3
    bfree = statfs_struct.f_bfree;
    if( bfree < 0L ) bfree = 0L;
    *avail_bytes = bfree * BLKSIZ;  /* SYSV */
    disk_space   = statfs_struct.f_blocks * BLKSIZ;
#else 
#if defined( ultrix )
    bfree = statfs_struct.fd_req.bfree;
    if( bfree < 0L ) bfree = 0L;
#else
    bfree = statfs_struct.f_bfree;
    if( bfree < 0L ) bfree = 0L;
    *avail_bytes = bfree * statfs_struct.f_bsize;
    disk_space   = statfs_struct.f_blocks * statfs_struct.f_bsize;
#endif /* ultrix */
#endif /* SVR3 */
#endif /* SVR4/!__DGUX__ */
#endif /* _IBMR2/linux/sun/__NeXT__ */
#endif /* WIN32 */
    
    if( total_disk_space )
    {
      *total_disk_space = disk_space;
    }
  }
  return( result );
}




int GetAvailBytes(long unsigned int *avail_bytes)
{
  return( GetDiskParameter( statistic.tree->name, 
			    NULL, 
			    avail_bytes, 
			    NULL 
			  ) 
        );
}



